import type { NormalizedCacheObject, ServerError } from '@apollo/client'
import { ApolloClient, ApolloLink, HttpLink, InMemoryCache, from } from '@apollo/client'
import { makeVar, useReactiveVar } from '@apollo/client'
import { onError } from '@apollo/client/link/error'
import ApolloLinkTimeout from 'apollo-link-timeout'
import { createUploadLink } from 'apollo-upload-client'
import fetch from 'cross-fetch'
import { concat, mergeDeepWith, pick, pipe, uniqBy } from 'ramda'

import { holiApiUrl } from '@holi/api/config'
import { HoliError } from '@holi/core/errors/classes/HoliError'
import { isErrorOfType, logError } from '@holi/core/errors/helpers'

import { alwaysReplacePolicy, indexAwareOffsetLimitPagination, nestedPaginationPolicy } from './cachePolicies'
import headerLink from './headerLink'

// The HOLI backend needs ~35s for a cold boot (which it does on a regular basis as we scale to zero), timeout respects that
const TIMEOUT_IN_MS = 60 * 1000 // 60 seconds

const timeoutLink = new ApolloLinkTimeout(TIMEOUT_IN_MS)
const unauthorizedErrorStatusVar = makeVar<boolean>(false)
export const setUnauthorizedErrorStatus = unauthorizedErrorStatusVar
export const useUnauthorizedErrorStatusVar = () => useReactiveVar(unauthorizedErrorStatusVar)

// There might be something fishy here, types dont match and if it is not the last link in the chain, throws the error "You are calling concat on a terminating link"
const uploadLink: ApolloLink = createUploadLink({ uri: holiApiUrl, credentials: 'include' })

const errorLink = onError(({ graphQLErrors, networkError }) => {
  // Do not log "not found" or "bad user input" errors
  if (graphQLErrors && isErrorOfType({ graphQLErrors }, ['NOT_FOUND', 'BAD_USER_INPUT'])) {
    return
  }
  if (graphQLErrors) {
    graphQLErrors.forEach(({ message, locations, path }) => {
      const errorMessage = `Message: ${message}, Location: ${locations}, Path: ${path}`
      logError(new HoliError(errorMessage), '[GraphQL Error]')
    })
  }
  if (networkError) {
    if ((networkError as ServerError)?.statusCode === 401) {
      setUnauthorizedErrorStatus(true)
    } else {
      logError(networkError, '[Network error]')
    }
  }
})

// Web authentication works using http-only cookies that are transparently added to requests by the Browser
// Mobile authentication works by adding the `X-Session-Token` header which takes its value from `SecureStorage`
// For mobile it is also necessary to explicitly set the Accept-Language header.
// `headerLink` is therefore only needed on Mobile (i.e. when defined)
const link = from([headerLink, timeoutLink, errorLink, uploadLink].filter((link) => !!link))

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const spacesKeyArgsFunction = (args: Record<string, any> | null) => {
  const { geolocation, topicId, topicSlug, skillIds, skillSlugs } = args || {}
  return JSON.stringify({
    topicId,
    topicSlug,
    skillIds,
    skillSlugs,
    geolocation: geolocation?.properties?.place_id,
  })
}

export const createCache = () =>
  new InMemoryCache({
    // eslint-disable-next-line prettier/prettier
    typePolicies: {
      User: {
        fields: {
          interests: alwaysReplacePolicy,
          skills: alwaysReplacePolicy,
          sdgs: alwaysReplacePolicy,
        },
      },
      InsightPaged: {
        fields: {
          data: indexAwareOffsetLimitPagination(),
        },
      },
      PostPaged: {
        fields: {
          data: indexAwareOffsetLimitPagination(),
        },
      },
      AppDonations_ProjectsResponse: {
        fields: {
          data: indexAwareOffsetLimitPagination(),
        },
      },
      AppVolunteering_EngagementsResponse: {
        fields: {
          data: indexAwareOffsetLimitPagination(),
        },
      },
      AppGoodNews_GoodNewsArticlesQueryResponse: {
        fields: {
          articles: {
            merge: (existing = [], incoming = []) =>
              pipe<unknown[][], unknown[], unknown[]>(concat, uniqBy(pick(['__ref'])))(existing, incoming),
            read: (existing) => existing,
          },
        },
      },
      Space: {
        fields: {
          members: alwaysReplacePolicy,
          tasks: alwaysReplacePolicy,
          sdgs: alwaysReplacePolicy,
          topics: alwaysReplacePolicy,
          links: alwaysReplacePolicy,
          collaborationTools: alwaysReplacePolicy,
        },
      },
      SpacePaged: {
        fields: {
          data: indexAwareOffsetLimitPagination(),
        },
      },
      SpaceMemberPaged: {
        fields: {
          data: nestedPaginationPolicy(['idOrName']),
        },
      },
      Query: {
        fields: {
          // here, we can specify e.g. pagination behavior, depending on the
          // query, see https://www.apollographql.com/docs/react/pagination/overview
          lastInsights: {
            keyArgs: [],
          },
          appDonations_projects: {
            keyArgs: ['location'],
          },
          appGoodNews_articles: {
            keyArgs: [],
          },
          appVolunteering_engagements: {
            keyArgs: ['location', 'category'],
          },
          spaces: {
            keyArgs: spacesKeyArgsFunction,
          },
          discoverSpaces: {
            keyArgs: spacesKeyArgsFunction,
          },
        },
      },
    },
  })

interface ApolloCreateClientOptions {
  readonly ssrMode: boolean
}

const isSSR = typeof window === 'undefined'

const API_URL = process.env.HOLI_API_URL
const httpLink = new HttpLink({ uri: API_URL, fetch })

const createApolloClient = ({ ssrMode }: ApolloCreateClientOptions) => {
  const clientLink = isSSR ? from([errorLink, httpLink]) : link

  return new ApolloClient({ ssrMode, link: clientLink, cache: createCache() })
}

// Web
/** This `let` variable can only be mutated by the initializeApolloSSRClient function. */
let apolloSSRClient: ApolloClient<NormalizedCacheObject> | null = null

type SSRClientInitState = Record<string, unknown> | null

export const initializeApolloSSRClient = (initState: SSRClientInitState = null) => {
  const client = apolloSSRClient ?? createApolloClient({ ssrMode: isSSR })

  if (initState) {
    const existingCache = client.extract()
    const merge = mergeDeepWith(concat)

    client.cache.restore(merge(existingCache, initState))
  }

  if (isSSR) return client
  if (!apolloSSRClient) apolloSSRClient = client

  return client
}

// Mobile
// TODO: rename to recognize its only for client-side JS
export const client = createApolloClient({ ssrMode: false })

export default client
